/*
 * ---------------------------------------------------------------------------------------------------------
 * Name : AlertsComponent class
 * Author : 91075
 * Purpose : Alert Messages Component
 * ----------------------------------------------------------------------------------------------------------
 */

import { Component, Input, OnInit, OnDestroy, SecurityContext } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import "rxjs/add/operator/takeWhile";
import { TranslateService } from 'ng2-translate';
import { BaseComponent } from './base.component';
import { LanguageService } from './language.service';
import { AlertMessageService } from './alert-message.service';
import { LoggerService } from './logger.service';
import { Helper } from './helper';
import { Settings } from './settings';

@Component({
    selector: 'alert-message',
    template: ` 
            <ng-container *ngIf="alertMessage && alertMessage.type && alertMessage.messages && alertMessage.messages.length > 0">
                <alert [type]="alertMessage.type">
                    <ng-container *ngFor="let msg of alertMessage.messages">
                        <span [innerHtml]="msg.value"></span>
                    </ng-container>
                </alert>
            </ng-container>
    `
})
export class AlertMessageComponent extends BaseComponent implements OnInit, OnDestroy {
    private alertMessage: AlertMessage;
    private _centralizedMessageHandling: boolean = true;
    private alive = true;

    @Input()
    set centralizedMessageHandling(value: boolean) {
        this._centralizedMessageHandling = value;
    }

    public constructor(public langService: LanguageService, 
            public translateService: TranslateService, 
            private alertMessageService: AlertMessageService,
            private logger: LoggerService,
            private sanitizer: DomSanitizer) {
        super(langService, translateService);
    }

    /**
     * Created a subscriber to observe AlertMessage when it fired
     * @return void
     */
    ngOnInit() {
        if (this._centralizedMessageHandling) {
            this.alertMessageService.alertMessagesObservable.takeWhile(() => this.alive).subscribe(alertMessages => {
                if (alertMessages) {
                    this.alertMessage = this.translateMessages(alertMessages);
                } else {
                    // clear old message
                    this.alertMessage = null;
                }
            });
        }
    }

    ngOnDestroy() {
        // to avoid memory leak
        this.alive = false;
    }

    /**
     * Translate all messages and also sanitize those messages
     * 
     * @param  {AlertMessage} alertMessages [description]
     * @param  {any}          parameters    [description]
     * @return {AlertMessage}               [description]
     */
    private translateMessages(alertMessages: AlertMessage, parameters?: any): AlertMessage {
        if (alertMessages) {
            // translate to msgValues by using msgKeys
            alertMessages.getMessages().forEach((messageInfo: MessageInformation, index) => {
                this.translate.get(messageInfo.key, parameters).subscribe(translated => {
                    // sanitize message
                    alertMessages.getMessages()[index].value = this.sanitizer.sanitize(SecurityContext.HTML, translated);
                });
            });
        }
        return alertMessages;
    }

    /**
     * Set alert message, should be used in case of decentralized message handling
     * 
     * @param {AlertMessage} alertMessage [description]
     */
    setAlertMessage(alertMessage: AlertMessage): void {
        if (alertMessage) {
            this.alertMessage = this.translateMessages(alertMessage);
        } else {
            this.alertMessage = null;
        }
    }

    /**
     * Clear old messages
     */
    clearAlertMessage(): void {
        this.alertMessage = null;
    }

    /**
     * Alert with custom alert message
     * 
     * @param {AlertMessage} alertMessages [description]
     * @param {any}          parameters    [description]
     */
    alert(alertMessages: AlertMessage, parameters?: any): void {
        this.alertMessage = this.translateMessages(alertMessages, parameters);
    }

    /**
     * Alert success message 
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertSuccess(msgKey: any, parameters?: any): void {
        this.alertMessage = this.translateMessages(new SuccessMessage(msgKey), parameters);
    }

    /**
     * Alert warning message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertWarning(msgKey: any, parameters?: any): void {
        this.alertMessage = this.translateMessages(new WarningMessage(msgKey), parameters);
    }

    /**
     * Alert danger message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertDanger(msgKey: any, parameters?: any): void {
        this.alertMessage = this.translateMessages(new DangerMessage(msgKey), parameters);
    }

    /**
     * Alert information message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertInfo(msgKey: any, parameters?: any): void {
        this.alertMessage = this.translateMessages(new InfoMessage(msgKey), parameters);
    }

    /**
     * Get the return code from received response, also alert responding message if any
     * 
     * @param  {any}    response   [description]
     * @param  {any}    parameters [description]
     * @return {number}            [description]
     */
    getReturnCode(response: any, parameters?: any): number {
        let returnCode: number = Settings.RC_GENERIC_ERROR;    // by default

        if (response && response.baseResponse && !isNaN(response.baseResponse.returnCode)) {
            returnCode = response.baseResponse.returnCode; 
            if (Helper.isSuccess(returnCode) || Helper.hasNoServiceImplementation(returnCode)) {
                return returnCode;
            } else {
                // define { key, value } of message information in here, bases on return code
                if (Helper.hasNoPermission(returnCode)) {   // return code of 201
                    this.alertDanger(Settings.MSG_KEY_HAS_NO_PERMISSION);
                } else {
                    this.alertDanger(Settings.MSG_KEY_GENERIC_ERROR);
                }
            }
        } else {
            this.alertDanger(Settings.MSG_KEY_GENERIC_ERROR);
        }

        if (this.logger.enabled) {
            this.logger.log("ReturnCode: ", returnCode);
        }
        return returnCode;
    }
}

export enum StateObject {
    SUCCESS = <any>'success',
    INFO = <any>'info',
    WARNING = <any>'warning',
    DANGER = <any>'danger',
}

export class MessageInformation {
    key: any;       // for ex: "application.message.system_error"
    value: any;     // for ex: "Please contact to Customer Service."

    constructor() {
        this.key = null;
        this.value = null;
    }
}

export class AlertMessage {
    // Alert type. Provides one of four bootstrap supported contextual classes: success, info, warning and danger
    protected type: StateObject;
    // message list will be displayed
    protected messages: Array<MessageInformation>;

    constructor() {
        this.initialize();
    }

    getType(): StateObject {
        return this.type;
    }

    getMessages(): Array<MessageInformation> {
        return this.messages;
    }

    initialize() {
        this.messages = [];
        this.type = StateObject.WARNING;
    }

    /**
     * Clear all messages
     */
    clearAllMessage(): void {
        this.messages = [];
    }
}

export class ValidationMessages extends AlertMessage {
    constructor() {
        super();
        this.type = StateObject.DANGER;
    }

    /**
     * Multi messages
     */
    setMessages(messages: Array<MessageInformation>): void {
        this.messages =  messages;
    }

    /**
     * Multi messages
     */
    addMessage(message: MessageInformation): void {
        this.messages.push(message);
    }

    setType(type: StateObject): void {
        this.type = type;
    }
}

export class ContextualMessage extends AlertMessage {
    constructor(msgKey: any) {
        super();
        let msg: MessageInformation = new MessageInformation();
        msg.key = msgKey;
        this.messages = [ msg ];
    }

    /**
     * Single message
     */
    setMessage(message: MessageInformation): void {
        this.messages =  [ message ];
    }
}

export class SuccessMessage extends ContextualMessage {
    constructor(msgKey: any) {
        super(msgKey);
        this.type = StateObject.SUCCESS;
    }
}

export class WarningMessage extends ContextualMessage {
    constructor(msgKey: any) {
        super(msgKey);
        this.type = StateObject.WARNING;
    }
}

export class InfoMessage extends ContextualMessage {
    constructor(msgKey: any) {
        super(msgKey);
        this.type = StateObject.INFO;
    }
}

export class DangerMessage extends ContextualMessage {
    constructor(msgKey: any) {
        super(msgKey);
        this.type = StateObject.DANGER;
    }
}

// export class AlertMessage {
//     // message list will be displayed
//     message: any;
//     // Alert type. Provides one of four bootstrap supported contextual classes: success, info, warning and danger
//     type: StateObject = StateObject.WARNING;
//     // If set, displays an inline "Close" button.
//     dismissible: boolean = false;
//     // Number in milliseconds, after which alert will be closed
//     dismissOnTimeout: number = undefined;
// }