/* ---------------------------------------------------------------------------------------------------------
 * Name : Alerts Service classs
 * Title : Alerts Service
 * Author : 91075
 * Creation Date : Mar 03, 2017
 * Purpose : Observe the Alert Messages
 * ----------------------------------------------------------------------------------------------------------
 */
import { Injectable } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Observable } from 'rxjs/Observable';
import { AlertMessage, SuccessMessage, WarningMessage, DangerMessage, InfoMessage } from './alert-message.component';
import { Helper } from './helper';
import { Settings } from './settings'
import { LoggerService } from './logger.service';

@Injectable()
export class AlertMessageService {
    constructor(private logger: LoggerService) { }

    private observer = new Subject<AlertMessage>();

    get alertMessagesObservable(): Observable<AlertMessage> {
        return this.observer.asObservable();
    }

    /**
     * Subject AlertMessages when it coming
     * @param alertMessages - AlertMessages
     * @return void
     */
    observeAlertMessage(alertMessages: AlertMessage, parameters?: any): void {
        this.observer.next(alertMessages);
    }

    /**
     * Clear old messages
     */
    clearAlertMessage(): void {
        this.observeAlertMessage(null);
    }

    /**
     * Alert with custom alert message
     * 
     * @param {AlertMessage} alertMessages [description]
     * @param {any}          parameters    [description]
     */
    alert(alertMessages: AlertMessage, parameters?: any): void {
        this.observeAlertMessage(alertMessages, parameters);
    }

    /**
     * Alert success message 
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertSuccess(msgKey: any, parameters?: any): void {
        this.observeAlertMessage(new SuccessMessage(msgKey), parameters);
    }

    /**
     * Alert warning message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertWarning(msgKey: any, parameters?: any): void {
        this.observeAlertMessage(new WarningMessage(msgKey), parameters);
    }

    /**
     * Alert danger message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertDanger(msgKey: any, parameters?: any): void {
        this.observeAlertMessage(new DangerMessage(msgKey), parameters);
    }

    /**
     * Alert information message
     * 
     * @param {any} msgKey     [description]
     * @param {any} parameters [description]
     */
    alertInfo(msgKey: any, parameters?: any): void {
        this.observeAlertMessage(new InfoMessage(msgKey), parameters);
    }

    /**
     * Get the return code from received response, also alert responding message if any
     * 
     * @param  {any}    response   [description]
     * @param  {any}    parameters [description]
     * @return {number}            [description]
     */
    getReturnCode(response: any, parameters?: any): number {
        let returnCode: number = Settings.RC_GENERIC_ERROR;    // by default

        if (response && response.baseResponse && !isNaN(response.baseResponse.returnCode)) {
            returnCode = response.baseResponse.returnCode; 
            if (Helper.isSuccess(returnCode) || Helper.hasNoServiceImplementation(returnCode)) {
                return returnCode;
            } else {
                // define { key, value } of message information in here, bases on return code
                if (Helper.hasNoPermission(returnCode)) {   // return code of 201
                    this.alertDanger(Settings.MSG_KEY_HAS_NO_PERMISSION);
                } else {
                    this.alertDanger(Settings.MSG_KEY_GENERIC_ERROR);
                }
            }
        } else {
            this.alertDanger(Settings.MSG_KEY_GENERIC_ERROR);
        }

        if (this.logger.enabled) {
            this.logger.log("ReturnCode: ", returnCode);
        }
        return returnCode;
    }
}