/*
* ---------------------------------------------------------------------------------------------------------
* Name : PaymentHelper methods
* Title : Export methods
* Author : 91070
* Creation Date : Jan 11, 2017
* Purpose : Define general helper for payment UI
* Functional Spec: SCC-89 Pay Bill w/ Credit Card (One-Time)
* ----------------------------------------------------------------------------------------------------------
*/
import { Injectable } from '@angular/core';
import { Settings } from './settings';
import * as moment from 'moment';

@Injectable()
export class Helper {

    /**
     * isEmpty method helper
     * @param
     *       firstObject : any
     * @return boolean true/false
     */
    static isEmpty(firstObject: any): boolean {
        return firstObject == null || Object.keys(firstObject).length === 0;
    }

    /**
     * isNumberNullOrZero method helper
     * @param
     *       firstObject : any
     * @return boolean true/false
     */
    static isNumberNullOrZero(firstObject: number): boolean {
        return firstObject == null || firstObject === 0;
    }

    /**
     * equals method helper
     * @param
     *       firstObject : any
     *       secondObject : any
     * @return boolean true/false
     */
    static equals(firstObject: any, secondObject: any): boolean {
        if (this.isEmpty(firstObject)) return false;
        if (this.isEmpty(secondObject)) return false;

        // Doing comparation
        return firstObject == secondObject;
    }

    /**
     * isStringEmpty method helper
     * @param
     *       inputValue : string
     * @return boolean true/false
     */
    static isStringEmpty(inputValue: string): boolean {
        return !inputValue || 0 === inputValue.length;
    }

    /**
     * isStringBlank method helper
     * @param
     *       inputValue : string
     * @return boolean true/false
     */
    static isStringBlank(inputValue: string): boolean {
        return !inputValue || /^\s*$/.test(inputValue);
    }

    /**
     * isBiggerThanZero method helper
     * @param
     *       inputValue : number
     * @return boolean true/false
     */
    static isBiggerThanZero(inputValue: number): boolean {
        return inputValue != null && inputValue > 0;
    }

    /**
     * equals method helper
     * @param
     *       firstObject : any
     *       secondObject : any
     * @return boolean true/false
     */
    static equalsIgnoreCase(firstString: string, secondString: string): boolean {
        if (firstString == null && secondString == null) return true;

        // Doing comparation
        return firstString.toUpperCase() === secondString.toUpperCase();
    }
    
    /**
     * Check if input object is null
     * @param
     *       inputObject : any
     * @return boolean true/false
     */
    static isNull(inputObject: any): boolean {
        return inputObject === null; 
    }

    /**
     * Check if return code is Success
     * @param {number} returnCode [description]
     */
    static isSuccess(returnCode: number): boolean {
        return returnCode === Settings.RC_SUCCESS;
    }

    /**
     * Check if return code is No Service Implmentation
     * @param {number} returnCode [description]
     */
    static hasNoServiceImplementation(returnCode: number): boolean {
        return returnCode === Settings.RC_NO_SERVICE_IMPLEMENTATION;
    }

    /**
     * Check if return code is unable to delete in look-aside table
     * @param returnCode 
     */
    static isUnableToDeleteRecordInLookAsideTable(returnCode: number): boolean {
        return returnCode === Settings.RC_UNABLE_DELETE_RECORD_IN_TABLE;
    }

    /**
     * Check if return code is type of no permission
     * @param returnCode 
     */
    public static hasNoPermission(returnCode: number): boolean {
        return returnCode === Settings.RC_NO_PERMISSION;
    }    

    /**
     * Convert string to date.
     * See more: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date#Conversion_getter
     * 
     * @param  {string} value [description]
     * @param  {any[]}  args  [description]
     * @return {string}       [description]
     */
    static converntToDate(value: string): Date {
        let parsedDate = Date.parse(value);
        if (isNaN(parsedDate)) {
            return null;
        } else {
            return new Date(parsedDate);
        }
    }

    /**
     * Convert string to Date, bases on originalFormatString
     * Ex: "00000000" to "0000-00-00"
     * 
     * @param  {string} value                [description]
     * @param  {string} originalFormatString [description]
     * @return {Date}                        [description]
     */
    static convertToDateString(value: string, originalFormatString: string): string {
        if (this.isStringBlank(value) || this.isStringBlank(originalFormatString)) {
            return "";
        }

        let formatValue: string = "";
        if (this.equalsIgnoreCase(Settings.DATE_STRING_DEFAULT_8, originalFormatString)
                && value.length === 8) {
            formatValue = value.replace(/(\d\d\d\d)(\d\d)(\d\d)/, '$1-$2-$3');
        }
        if (this.equalsIgnoreCase(Settings.DATE_STRING_DEFAULT_10, originalFormatString)
                && value.length === 10) {
            formatValue = value;
        }

        return formatValue;
    }

    /**
     * Calculate the number of days between 2 dates
     * 
     * @param  {Date}   firstDate  [description]
     * @param  {Date}   secondDate [description]
     * @return {number}            -1: invalid inputs, others: different days between input dates
     */
    static calculateDifferentDays(firstDate: Date, secondDate: Date): number {
        if (Helper.isNull(firstDate) || Helper.isNull(secondDate)) {
            return -1;
        }

        let oneDay = 24*60*60*1000; // hours*minutes*seconds*milliseconds

        return Math.round(Math.abs((firstDate.getTime() - secondDate.getTime())/(oneDay)));
    }

    /**
     * Convert string to Moment(aka Date), bases on originalFormatString
     * Ex: "00000000" to new Date("0000-00-00")
     * 
     * @param  {string}        value                [description]
     * @param  {string}        originalFormatString [description]
     * @return {moment.Moment}                      [description]
     */
    static convertDateStringToMoment(value: string, originalFormatString: string): moment.Moment {
        let formatValue: string = this.convertToDateString(value, originalFormatString);

        return this.isStringBlank(formatValue) ? null : moment(formatValue);
    }

    static convertMomentToDateString(value: moment.Moment, formatString: string): string {
        if (this.isNull(value)) {
            return "";
        }
        if (this.isStringBlank(formatString)) {
            formatString = Settings.DATE_STRING_DEFAULT_8;
        }

        // first, convert to mm/dd/yyyy
        let formatValue: string = value.clone().format(Settings.DATE_DEFAULT_FORMAT);
        console.log("formatValue: ", formatValue);
             
        if (this.equalsIgnoreCase(Settings.DATE_STRING_DEFAULT_8, formatString)) {
            // then, convert to yyyymmdd
            formatValue = formatValue.replace(/(\d\d)\/(\d\d)\/(\d\d\d\d)/, '$3$1$2');
        } else if (this.equalsIgnoreCase(Settings.DATE_STRING_DEFAULT_10, formatString)) {
            formatValue = formatValue.replace(/(\d\d)\/(\d\d)\/(\d\d\d\d)/, '$3-$1-$2');
        } else {
            // return date default format
        }

        return formatValue;
    }

    public static sleep(ms):Promise<number> {
        return new Promise(resolve=>setTimeout(resolve, ms));
    }

    public static compareObjects(a:any, b:any): number {
        if((a==null||a==undefined)&&b) {
            return -1;
        }
        if(a&&(b==null||b==undefined)) {
            return 1;
        }
        return 0;
    }

    public static isCompanyMGE(companyType: string): boolean {
        return this.equals(companyType, Settings.COMPANY_MGE);
    }

    public static isCompanyAGC(companyType: string): boolean {
        return this.equals(companyType, Settings.COMPANY_AGC);
    }

    public static isCompanyLGC(companyType: string): boolean {
        return this.equals(companyType, Settings.COMPANY_LGC);
    }

    /**
     * Check if the customer is Alabama
     * 
     * @param companyType 
     */
    public static isAL(companyType: string): boolean {
        return this.isCompanyAGC(companyType);
    }

    /**
     * Check if the customer is Missouri
     * 
     * @param companyType 
     */
    public static isMO(companyType: string): boolean {
        return this.isCompanyMGE(companyType) || this.isCompanyLGC(companyType);
    }
}
