/*
 * ---------------------------------------------------------------------------------------------------------
 * Name : AlertsComponent class
 * Author : 91075
 * Purpose : Alert Messages Component
 * ----------------------------------------------------------------------------------------------------------
 */

import { Component, Input, Output, SecurityContext, ViewChild, EventEmitter } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { TranslateService } from 'ng2-translate';
import { ModalDirective } from 'ng2-bootstrap';
import { BaseComponent } from './base.component';
import { LanguageService } from './language.service';
import { LoggerService } from './logger.service';
import { Helper } from './helper';
import { Settings } from './settings';

@Component({
    selector: 'modal-confirmation',
    template: ` 
        <div bsModal #staticModal="bs-modal" class="modal fade" tabindex="-1" role="dialog" aria-labelledby="mySmallModalLabel" aria-hidden="true" [config]="{backdrop: 'static'}" style="display:none">
            <div class="modal-dialog modal-lg spire-modal-pos-r">
                <div class="modal-content">
					<ng-container *ngIf="modalObject">
					<div class="modal-header">
                        <ng-container *ngIf="modalObject.headerTitle">
						<h4 class="modal-title pull-left">{{ modalObject.headerTitle | translate }}</h4>
                        </ng-container>
						<button type="button" class="close pull-right" (click)="staticModal.hide()" aria-label="Close">
						<span aria-hidden="true">&times;</span>
						</button>
					</div>
					<div class="modal-body">
                        <ng-container *ngIf="modalObject.bodyTitle">
						<div class="row pin-modal-confirmation-row">
							<div class="col-md-12 text-center">
								<h2>{{ modalObject.bodyTitle | translate }}</h2>
							</div>
						</div>
						<br/>
                        </ng-container>
						<div class="row pin-modal-confirmation-row">
							<div class="col-md-12" [innerHTML]="modalObject.bodyHTML"></div>
						</div>
						<br/>			
						<div class="row pin-modal-confirmation-row">
							<div class="col-md-12 text-center">
								<button type="button" class="{{ modalObject.rightButton.cssClass }}" (click)="onClickRightButton()">{{ modalObject.rightButton.label | translate }}</button>
								<button type="button" class="{{ modalObject.leftButton.cssClass }}" (click)="onClickLeftButton()">{{ modalObject.leftButton.label | translate }}</button>
							</div>
						</div>
						<br/>
					</div>
					</ng-container>
                </div>
            </div>
        </div>
    `
})
export class ModalConfirmationComponent extends BaseComponent {
    private _modalObject: ModalObject = {};
    
    @Input() 
    set modalObject(modalObject: ModalObject) {
        if (modalObject) {
            this.translate.get(modalObject.bodyContent).subscribe(translated => {
                // sanitize message
                modalObject.bodyHTML = this.sanitizer.sanitize(SecurityContext.HTML, translated);
            });
            this._modalObject = modalObject;
            // use default css class if needed
            if (this._modalObject.rightButton && !this._modalObject.rightButton.cssClass) {
                this._modalObject.rightButton.cssClass = "btn btn-primary btn-lg pin-modal-confirmation-btn";
            }
            if (this._modalObject.leftButton && !this._modalObject.leftButton.cssClass) {
                this._modalObject.leftButton.cssClass = "btn btn-secondary btn-lg pin-modal-confirmation-btn";
            }
        }
    }
    get modalObject(): ModalObject { return this._modalObject; }

    @Output() modalConfirmationEvent: EventEmitter<ModalReturnObject> = new EventEmitter();
    @ViewChild('staticModal') staticModal: ModalDirective;

    public constructor(public langService: LanguageService, 
            public translateService: TranslateService, 
            private logger: LoggerService,
            private sanitizer: DomSanitizer) {
        super(langService, translateService);
    }

    /**
     * Return right button and responding data if it is selected 
     */
    onClickRightButton() : void {
        let modalReturnObject: ModalReturnObject = {};
        modalReturnObject.whichOneSelected = ModalSelectedButton.RIGHT;
        modalReturnObject.data = this.modalObject.data;
        this.modalConfirmationEvent.emit(modalReturnObject);
        this.close();
    }

    /**
     * Return left button and responding data if it is selected 
     */
    onClickLeftButton() : void {
        let modalReturnObject: ModalReturnObject = {};
        modalReturnObject.whichOneSelected = ModalSelectedButton.LEFT;
        modalReturnObject.data = this.modalObject.data;
        this.modalConfirmationEvent.emit(modalReturnObject);
        this.close();
    }

    /**
     * [close description]
     */
    close() : void {
        this.staticModal.hide();
    }

    /**
     * [show description]
     */
    show() : void {
        this.staticModal.show();
    }

}

export interface ModalButton {
    label?: string;
    cssClass?: string;
}

export interface ModalObject {
    headerTitle?: any;  // for ex: "user-pin.modal-confirmation.revoke_pin_header_title"
    bodyTitle?: any;    // for ex: "user-pin.modal-confirmation.revoke_pin_body_title"
    bodyContent?: any;  // for ex: "user-pin.modal-confirmation.revoke_pin_body_html"
    bodyHTML?: any;     // translated and sanitized message HTML
    rightButton?: ModalButton;
    leftButton?: ModalButton;
    data?: any;
}

export interface ModalReturnObject {
    whichOneSelected?: ModalSelectedButton;
    data?: any;
}

export enum ModalSelectedButton {
    RIGHT = <any>"RIGHT",
    LEFT = <any>"LEFT"
}

// NOTE: should add new style "pos-r" (or "spire-modal-pos-r") to set position to relative and let modal is displayed in center